#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#    Copyright © Manoel Vilela 2016
#
#    @project: Decorating
#     @author: Manoel Vilela
#      @email: manoel_vilela@engineer.com
#
#
#                  ..        .;-:!!>>!!:--;..  .
#               .     .-:!>7?CO$QQQQ$OC?:--::-.....
#             .   .-!7?CO$QQQQQ$$$OCCCC?!:--:>>:;;;...
#           .  .->CCCC>?OO$$OO$$$$$$O$$$O?7>:>C$C7!:-;;.  .
#         .  .!?O$Q$$OOCC?CCO$QQQQQQ$$QQQ$$$OO$QNHH$C?>!:;  .
#        . .!OQQQHHHQQQQQ$$QQQQHQHQQQQQHQQHHHHHHHNNNHHQ$OC>.   .
#       . ;CQHHHHHHHHHHHHHHHHHHHHHHHQHHHHHHNNNNNNNNHNHNNHHH$!;
#        :OHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHNNNNNNNNNHHHHNNHNNO:-.
#       !OHNNNNNHHHHHHHHHHHHHHHHHHHHHHHHNNHHNNNNNNHHHHHHNHNNNH>;;.
#    . !OHNHHHHNHHHHHHHHHHHHHHHNNNNNNNHHHNNNNNNNNHNMNNNNHNNNHNN7  .
#     -OQNHNNNNNNHNHHNNHHNNNNNNNNNNNNNNHHHNNNNNHHNQ>!>OHNNHNNHHN$;
#    .?$HHNNNNNNNNNNHHNHHNNNNNNNNNNNNNNNNNNNNNNHNQ.;>:::CNNHNNHHNQ!.
#  . :$HNNNNNNHHNNHHNNNNHNNNNNNNNNNHHNHNNNNNNHHHN? !;  . -ONHNNHNO!-....
#   .?QHNNNNNNNNNNNNNNNNNNNNNNNNNHHNNHNHHNNNNNNNNC .;->!:..QNHHHN$!:-;.
# . :$HNNNHNNNNNNNNNNNNNNNNNNNNHHNM$QNHNNHNNNHQQNO  :$O>!- CNHHHHN$O7...
# . !QHNNNNNNNNNNNNNNNNNNNHHNNHNNO>;;?:7QNNNQQHQQH! .O7!; :HHHHHNNNH:...
# . !HHNNNNNNNNNNNNNNNNNNNHHHHH$>;.-!7C?O$HOHNNNNO?; -... CMHNNNHQH> ...
# . !HHNNNNNNNNNNNNNNNNNNNHHNN$>77OHNNHHQ7OC$QHHQ>;..  ..;$QOCOQOO!.....
# . !NNNNNNNNNNNNNNNNHHHHNHNNHC$CHO?Q$QQ? ?HQHQ$O$- .. .:??!-. :>- .....
#   .QNHNNNNNNNNNNNNNNNNNHHHN$!:-?; :7!.  ;>7NNNHQ7 .   .;;;..;:.......
#    ONHNNNNNNNNNNNNHO$HHMNNM?     .        .!HNHQO.      ;-.-:;....
#  . 7NHHNHNNNNNNNNHN?:!;>7:!!.      ..    .. 7NHQQ: .  ;;::>:;;.....
#    -!HNHHNNNNNNNNNNQ?>:;.   .             . .QNQQ? . ;>7C?:::.........
#      7NHNNHNNNNNHNQCCO$Q$O:.               . ?MHQ$.  >QO>!>7;.........
#    . .QHNNNNNHHHHHH$QHQNQ$>...             . >NHHN7 ;QO!>CC-..........
#       :-$HO$ONHHNHNNHO->CC; .;.            . 7NHHH$;?$>7O$:.......  ..
#         -$:--QHNQN>CMNQ:;--;;;....      .... ?NHNQ!OHCOOQ>;;;.....   .
#        . 7:  ;OH>?;.77-  ;;..:-      .....   ONHNQCHHHQHQ!--......  ..
#          .. . --.;;.       . ;O>:; .;........>$NNHNNNHHHH?-.... .   ..
#                ;..  ......;.;->>:::--;; .;..!>CNNQQNHNHNQ!....
#             . ;:. .......... .;:->?   -: .-.!HHHN7>NHHNH>;...      .
#              .!; ...   ........;!ONC>>C$7?O>7QNHNQ?HNNNO:...         .
#             :C!.;.   .... ...;-!?$NNNNNNNNNNNHHNHNNHHNQ7;...      ....
#            7$O-.. .        ...!$QNNHHHHHHHHHHHHNNHHNNQC-...     ......
#          .?Q$:......       ...-$NNNNHHHHNHNNNHHNNNNNQC>;..     .......


"""
    This is another LOL-zone

    LOOOOOOOOOOOOOL ART
"""
from __future__ import unicode_literals

# START
#     /\O    |    _O    |      O
#      /\/   |   //|_   |     /_
#     /\     |    |     |     |\
#    /  \    |   /|     |    / |
#  LOL  LOL  |   LLOL   |  LOLLOL
# -----------+----------+-----------


ARROWS = "←↖↑↗→↘↓↙"
BALLS = ".oO@*"
BRAILY = "⣾⣽⣻⢿⡿⣟⣯"
CIRCLE1 = "◐◓◑◒"
CIRCLE2 = "◴◷◶◵"
CYCLE = "|/-\\"
EYES = ["◡◡" "⊙⊙" "◠◠"]
HPULSE = "▉▊▋▌▍▎▏▎▍▌▋▊▉"
PIPES = "┤┘┴└├┌┬┐"
ROTATING = "▖▘▝▗"
SQUARES = "◰◳◲◱"
TRIANGULES = "◢◣◤◥"
VPULSE = '▁▂▃▄▅▆▇▆▅▄▃▁'
WAVE = "⠁⠂⠄⡀⢀⠠⠐⠈"


# END
#     /\O    |    _O    |      O
#      /\/   |   //|_   |     /_
#     /\     |    |     |     |\
#    /  \    |   /|     |    / |
#  LOL  LOL  |   LLOL   |  LOLLOL
# -----------+----------+-----------
